@@ -65,6 +65,7 @@ function ViewModel() {
     self.statusInfo.downloaddirspeed = ko.observable();
     self.statusInfo.completedir = ko.observable();
     self.statusInfo.completedirspeed = ko.observable();
+    self.statusInfo.internetbandwidth = ko.observable();
 
     /***
         Dynamic functions
@@ -96,7 +97,7 @@ function ViewModel() {
 
     // Dynamic speed text function
     self.speedText = ko.pureComputed(function() {
-        return self.speed() + ' ' + (self.speedMetrics[self.speedMetric()] ? self.speedMetrics[self.speedMetric()] : "KB/s");
+        return self.speed() + ' ' + (self.speedMetrics[self.speedMetric()] ? self.speedMetrics[self.speedMetric()] : "B/s");
     });
 
     // Dynamic icon
@@ -359,19 +360,6 @@ function ViewModel() {
             return;
         }
 
-        /**
-            Do first load with start-data
-            Only works when the server knows the settings!
-        **/
-        if(glitterPreLoadHistory && self.useGlobalOptions()) {
-            self.updateQueue(glitterPreLoadQueue);
-            self.updateHistory(glitterPreLoadHistory);
-            glitterPreLoadQueue = undefined;
-            glitterPreLoadHistory = undefined;
-            self.setNextUpdate()
-            return;
-        }
-
         /**
             Full refresh
         **/
@@ -680,6 +668,7 @@ function ViewModel() {
             mode: "addurl",
             name: $(form.nzbURL).val(),
             nzbname: $('#nzbname').val(),
+            password: $('#password').val(),
             script: $('#modal-add-nzb select[name="Post-processing"]').val(),
             priority: $('#modal-add-nzb select[name="Priority"]').val(),
             pp: $('#modal-add-nzb select[name="Processing"]').val()
@@ -719,6 +708,7 @@ function ViewModel() {
         data.append("name", file);
         data.append("mode", "addfile");
         data.append("nzbname", $('#nzbname').val());
+        data.append("password", $('#password').val());
         data.append("script", $('#modal-add-nzb select[name="Post-processing"]').val())
         data.append("priority", $('#modal-add-nzb select[name="Priority"]').val())
         data.append("apikey", apiKey);
@@ -729,7 +719,7 @@ function ViewModel() {
 
         // Add this one
         $.ajax({
-            url: "./tapi",
+            url: "./api",
             type: "POST",
             cache: false,
             processData: false,
@@ -776,6 +766,7 @@ function ViewModel() {
                 self.statusInfo.downloaddirspeed(data.status.downloaddirspeed)
                 self.statusInfo.completedir(data.status.completedir)
                 self.statusInfo.completedirspeed(data.status.completedirspeed)
+                self.statusInfo.internetbandwidth(data.status.internetbandwidth)
                 self.statusInfo.dnslookup(data.status.dnslookup)
                 self.statusInfo.localipv4(data.status.localipv4)
                 self.statusInfo.publicipv4(data.status.publicipv4)
@@ -849,6 +840,30 @@ function ViewModel() {
         })
     }
 
+    // Download a test-NZB
+    self.testDownload = function(data, event) {
+        var nzbSize = $(event.target).data('size')
+
+        // Maybe it was a click on the icon?
+        if(nzbSize == undefined) {
+            nzbSize = $(event.target.parentElement).data('size')
+        }
+
+        // Build request
+        var theCall = {
+            mode: "addurl",
+            name: "https://sabnzbd.org/tests/test_download_" + nzbSize + ".nzb",
+            priority: self.queue.priorityName["Force"]
+        }
+
+        // Add
+        callAPI(theCall).then(function(r) {
+            // Hide and reset/refresh
+            self.refresh()
+            $("#modal-options").modal("hide");
+        });
+    }
+
     // Unblock server
     self.unblockServer = function(servername) {
         callSpecialAPI("./status/unblock_server/", {
