@@ -25,6 +25,7 @@ jQuery(document).ready(function($){
   $("#help").colorbox({ inline:true, href:"#help_modal", title:$("#help").text(),
     innerWidth:"375px", innerHeight:"350px", initialWidth:"375px", initialHeight:"350px", speed:0, opacity:0.7
   });
+  $(".show_qrcode").colorbox({ photo:true, innerHeight:"300px", innerWidth:"300px", speed:0, opacity: 0.7, scrolling:false });
 
   // jqueryui tabs/buttons
   $('.juiButton').button();
@@ -121,9 +122,9 @@ jQuery(document).ready(function($){
   break;
 
 
-  case 'Connections':
+  case 'Status':
     $('#logging_level').change(function(event){
-      window.location = './change_loglevel?loglevel='+$(event.target).val()+'&session='+apikey;
+      window.location = './change_loglevel?loglevel='+$(event.target).val()+'&apikey='+apikey;
     });
     break;
 
@@ -133,7 +134,7 @@ jQuery(document).ready(function($){
       if (confirm($(this).attr('rel'))) {
         $.ajax({
           type: "POST",
-          url: "../../tapi",
+          url: "../../api",
           data: {mode:'config', name:'set_apikey', apikey: $('#apikey').val()},
           success: function(msg){
             $('#apikey,#session').val(msg);
@@ -146,7 +147,7 @@ jQuery(document).ready(function($){
       if (confirm($(this).attr('rel'))) {
         $.ajax({
           type: "POST",
-          url: "../../tapi",
+          url: "../../api",
           data: {mode:'config', name:'set_nzbkey', apikey: $('#apikey').val()},
           success: function(msg){
             $('#nzbkey,#session').val(msg);
@@ -165,7 +166,7 @@ jQuery(document).ready(function($){
       $(event.target).next('span').addClass('loading');
       $.ajax({
         type: "POST",
-        url: "../../tapi",
+        url: "../../api",
         data: "mode=config&name=test_server&"+ $(event.target).parents('form:first').serialize() +"&apikey="+$('#apikey').val(),
         success: function(msg){
           alert(msg);
@@ -178,11 +179,16 @@ jQuery(document).ready(function($){
         $(event.target).parents('form:first').attr('action','delServer').submit();
       return false;
     });
+    $('form .clrServer').click(function(event){ // clear server
+      if(confirm($(event.target).attr('rel')))
+        $(event.target).parents('form:first').attr('action','clrServer').submit();
+      return false;
+    });
     break;
 
   case 'Categories':
     $(':button').click(function(event){ // delete category
-      window.location="delete/?name="+$(event.target).attr('name')+'&session='+apikey;
+      window.location="delete/?name="+$(event.target).attr('name')+'&apikey='+apikey;
     });
     break;
 
@@ -190,7 +196,7 @@ jQuery(document).ready(function($){
 
   $('.toggleFeedCheckbox').click(function(){  // enable/disable feed
     window.onbeforeunload = null; // lose data?
-    this.form.action='toggle_rss_feed?session=$session';
+    this.form.action='toggle_rss_feed?apikey=$apikey';
     this.form.submit();
     return false;
   });
@@ -207,9 +213,9 @@ jQuery(document).ready(function($){
     break;
 
   case 'Index Sites':
-    $('#getBookmarks').click(function(){ window.location='getBookmarks?session='+apikey; });
-    $('#hideBookmarks').click(function(){ window.location='hideBookmarks?session='+apikey; });
-    $('#showBookmarks').click(function(){ window.location='showBookmarks?session='+apikey; });
+    $('#getBookmarks').click(function(){ window.location='getBookmarks?apikey='+apikey; });
+    $('#hideBookmarks').click(function(){ window.location='hideBookmarks?apikey='+apikey; });
+    $('#showBookmarks').click(function(){ window.location='showBookmarks?apikey='+apikey; });
     break;
 
   case 'Sorting':
