@@ -1,5 +1,5 @@
-#!/usr/bin/python -OO
-# Copyright 2007-2018 The SABnzbd-Team <team@sabnzbd.org>
+#!/usr/bin/python3 -OO
+# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,36 +18,43 @@
 """
 sabnzbd.par2file - All par2-related functionality
 """
-import os
-import logging
-import re
 import hashlib
+import logging
+import os
+import re
 import struct
 
+from sabnzbd.encoding import correct_unknown_encoding
 
-PROBABLY_PAR2_RE = re.compile(r'(.*)\.vol(\d*)[\+\-](\d*)\.par2', re.I)
-PAR_ID = "PAR2\x00PKT"
-PAR_RECOVERY_ID = "RecvSlic"
+PROBABLY_PAR2_RE = re.compile(r"(.*)\.vol(\d*)[\+\-](\d*)\.par2", re.I)
+PAR_PKT_ID = b"PAR2\x00PKT"
+PAR_FILE_ID = b"PAR 2.0\x00FileDesc"
+PAR_CREATOR_ID = b"PAR 2.0\x00Creator"
+PAR_RECOVERY_ID = b"RecvSlic"
 
 
 def is_parfile(filename):
-    """ Check quickly whether file has par2 signature """
-    try:
-        with open(filename, "rb") as f:
-            buf = f.read(8)
-            return buf.startswith(PAR_ID)
-    except:
-        pass
+    """Check quickly whether file has par2 signature
+    or if the filename has '.par2' in it
+    """
+    if os.path.exists(filename):
+        try:
+            with open(filename, "rb") as f:
+                buf = f.read(8)
+                return buf.startswith(PAR_PKT_ID)
+        except:
+            pass
+    elif ".par2" in filename.lower():
+        return True
     return False
 
 
 def analyse_par2(name, filepath=None):
-    """ Check if file is a par2-file and determine vol/block
-        return setname, vol, block
-        setname is empty when not a par2 file
+    """Check if file is a par2-file and determine vol/block
+    return setname, vol, block
+    setname is empty when not a par2 file
     """
     name = name.strip()
-    setname = None
     vol = block = 0
     m = PROBABLY_PAR2_RE.search(name)
     if m:
@@ -75,51 +82,48 @@ def analyse_par2(name, filepath=None):
     return setname, vol, block
 
 
-def parse_par2_file(nzf, fname):
-    """ Get the hash table and the first-16k hash table from a PAR2 file
-        Return as dictionary, indexed on names or hashes for the first-16 table
-        For a full description of the par2 specification, visit:
-        http://parchive.sourceforge.net/docs/specifications/parity-volume-spec/article-spec.html
+def parse_par2_file(fname, md5of16k):
+    """Get the hash table and the first-16k hash table from a PAR2 file
+    Return as dictionary, indexed on names or hashes for the first-16 table
+    The input md5of16k is modified in place and thus not returned!
+
+    For a full description of the par2 specification, visit:
+    http://parchive.sourceforge.net/docs/specifications/parity-volume-spec/article-spec.html
     """
     table = {}
     duplicates16k = []
 
     try:
-        f = open(fname, 'rb')
-    except:
-        return table
-
-    try:
-        header = f.read(8)
-        while header:
-            name, hash, hash16k = parse_par2_file_packet(f, header)
-            if name:
-                table[name] = hash
-                if hash16k not in nzf.nzo.md5of16k:
-                    nzf.nzo.md5of16k[hash16k] = name
-                elif nzf.nzo.md5of16k[hash16k] != name:
-                    # Not unique and not already linked to this file
-                    # Remove to avoid false-renames
-                    duplicates16k.append(hash16k)
-
+        with open(fname, "rb") as f:
             header = f.read(8)
+            while header:
+                name, filehash, hash16k = parse_par2_file_packet(f, header)
+                if name:
+                    table[name] = filehash
+                    if hash16k not in md5of16k:
+                        md5of16k[hash16k] = name
+                    elif md5of16k[hash16k] != name:
+                        # Not unique and not already linked to this file
+                        # Remove to avoid false-renames
+                        duplicates16k.append(hash16k)
+
+                header = f.read(8)
 
     except (struct.error, IndexError):
         logging.info('Cannot use corrupt par2 file for QuickCheck, "%s"', fname)
-        logging.info('Traceback: ', exc_info=True)
+        logging.debug("Traceback: ", exc_info=True)
         table = {}
     except:
-        logging.debug('QuickCheck parser crashed in file %s', fname)
-        logging.info('Traceback: ', exc_info=True)
+        logging.info("QuickCheck parser crashed in file %s", fname)
+        logging.debug("Traceback: ", exc_info=True)
         table = {}
-    f.close()
 
     # Have to remove duplicates at the end to make sure
     # no trace is left in case of multi-duplicates
     for hash16k in duplicates16k:
-        if hash16k in nzf.nzo.md5of16k:
-            old_name = nzf.nzo.md5of16k.pop(hash16k)
-            logging.debug('Par2-16k signature of %s not unique, discarding', old_name)
+        if hash16k in md5of16k:
+            old_name = md5of16k.pop(hash16k)
+            logging.debug("Par2-16k signature of %s not unique, discarding", old_name)
 
     return table
 
@@ -129,19 +133,19 @@ def parse_par2_file_packet(f, header):
 
     nothing = None, None, None
 
-    if header != PAR_ID:
+    if header != PAR_PKT_ID:
         return nothing
 
     # Length must be multiple of 4 and at least 20
-    len = struct.unpack('<Q', f.read(8))[0]
-    if int(len / 4) * 4 != len or len < 20:
+    pack_len = struct.unpack("<Q", f.read(8))[0]
+    if int(pack_len / 4) * 4 != pack_len or pack_len < 20:
         return nothing
 
     # Next 16 bytes is md5sum of this packet
     md5sum = f.read(16)
 
     # Read and check the data
-    data = f.read(len - 32)
+    data = f.read(pack_len - 32)
     md5 = hashlib.md5()
     md5.update(data)
     if md5sum != md5.digest():
@@ -156,11 +160,16 @@ def parse_par2_file_packet(f, header):
     # xx : Name (multiple of 4, padded with \0 if needed) **
 
     # See if it's the right packet and get name + hash
-    for offset in range(0, len, 8):
-        if data[offset:offset + 16] == "PAR 2.0\0FileDesc":
-            hash = data[offset + 32:offset + 48]
-            hash16k = data[offset + 48:offset + 64]
-            filename = data[offset + 72:].strip('\0')
-            return filename, hash, hash16k
+    for offset in range(0, pack_len, 8):
+        if data[offset : offset + 16] == PAR_FILE_ID:
+            filehash = data[offset + 32 : offset + 48]
+            hash16k = data[offset + 48 : offset + 64]
+            filename = correct_unknown_encoding(data[offset + 72 :].strip(b"\0"))
+            return filename, filehash, hash16k
+        elif data[offset : offset + 15] == PAR_CREATOR_ID:
+            # From here until the end is the creator-text
+            # Useful in case of bugs in the par2-creating software
+            par2creator = data[offset + 16 :].strip(b"\0")  # Remove any trailing \0
+            logging.debug("Par2-creator of %s is: %s", os.path.basename(f.name), correct_unknown_encoding(par2creator))
 
     return nothing
